"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const fs_1 = tslib_1.__importDefault(require("fs"));
const path_1 = tslib_1.__importDefault(require("path"));
const partition_1 = tslib_1.__importDefault(require("lodash/partition"));
const listr_1 = tslib_1.__importDefault(require("listr"));
const prompts_1 = require("./prompts");
const constants_1 = require("./constants");
const logger_1 = require("./logger");
const utils = tslib_1.__importStar(require("./utils"));
const kyc_services_1 = require("./kyc-services");
exports.deleteServicesStack = ({ client, profile, stackName }) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const servicesStackId = yield kyc_services_1.getStackId(client, stackName);
    if (servicesStackId) {
        logger_1.logger.info(`deleting kyc services stack: ${servicesStackId} (ETA: 5-10 minutes)`);
        yield utils.deleteStack(client, { StackName: servicesStackId });
        yield utils.awaitStackDelete(client, servicesStackId);
        logger_1.logger.info(`deleted kyc services stack: ${servicesStackId}`);
    }
    logger_1.logger.info('Note: it may take a few minutes for your stack to be deleted');
});
exports.destroy = (opts) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const { client, stackName, profile } = opts;
    yield prompts_1.confirmOrAbort(`DESTROY REMOTE MYCLOUD ${stackName}?? There's no undo for this one!`);
    yield prompts_1.confirmOrAbort(`Are you REALLY REALLY sure you want to MURDER ${stackName}?`);
    let buckets = yield utils.listStackBucketIds(client, stackName);
    buckets.forEach(id => logger_1.logger.info(id));
    yield prompts_1.confirmOrAbort('Delete these buckets?');
    const promiseDeleteServices = exports.deleteServicesStack(opts);
    const [big, small] = partition_1.default(buckets, id => constants_1.BIG_BUCKETS.find(logical => id.includes(logical.toLowerCase())));
    for (const id of small) {
        logger_1.logger.info(`emptying and deleting: ${id}`);
        yield utils.destroyBucket(client, id);
    }
    yield new listr_1.default([
        {
            title: 'disabling termination protection',
            task: (ctx) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield utils.disableStackTerminationProtection(client, stackName);
            })
        },
        {
            title: 'deleting primary stack',
            task: (ctx) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield utils.deleteStack(client, { StackName: stackName });
                yield utils.wait(5000);
                try {
                    yield utils.awaitStackDelete(client, stackName);
                }
                catch (err) {
                    yield utils.deleteStack(client, { StackName: stackName, RetainResources: constants_1.BIG_BUCKETS });
                }
            })
        },
    ]).run();
    yield promiseDeleteServices;
    yield Promise.all(big.map(id => utils.markBucketForDeletion(client, id)));
    const cleanupScriptPath = path_1.default.relative(process.cwd(), exports.createCleanupBucketsScript({ buckets: big, profile }));
    logger_1.logger.info(`The following buckets are too large to delete directly:
${big.join('\n')}

Instead, I've marked them for deletion by S3. They should be emptied within a day or so

After that you can delete them from the console or with this little script I created for you: ${cleanupScriptPath}
`);
});
exports.createCleanupBucketsScript = ({ buckets, profile }) => {
    const cleanupScript = fs_1.default.existsSync(path_1.default.resolve(process.cwd(), 'cleanup-buckets.sh'))
        ? `cleanup-buckets-${Date.now()}.sh`
        : `cleanup-buckets.sh`;
    const delBuckets = buckets.map(name => getDeleteBucketLine({ name, profile }));
    const scriptBody = `
#!/bin/bash

${delBuckets.join('\n')}
`;
    const scriptPath = path_1.default.resolve(process.cwd(), cleanupScript);
    fs_1.default.writeFileSync(scriptPath, scriptBody);
    fs_1.default.chmodSync(scriptPath, '0755');
    return scriptPath;
};
const getDeleteBucketLine = ({ name, profile }) => {
    let line = 'aws ';
    if (profile) {
        line += `--profile ${profile} `;
    }
    line += `s3 rb "s3://${name}"`;
    return line;
};
//# sourceMappingURL=destroy.js.map